// $Id: PrefixQuery.java,v 1.2 2002/11/24 17:43:03 mike Exp $

package org.z3950.zoom;
import com.k_int.IR.IRQuery;
import com.k_int.IR.QueryModels.PrefixString;

/**
 * Represents a Prefix query to be submitted to a Z39.50 Server.
 * <P>
 * <B>Synopsis:</B>
 * <P>
 * <PRE>
 *	Query q = new PrefixQuery("@and @attr 1=4 minerals " +
 *	                          "@or @attr 1=62 location @attr 1=62 place");
 *	ResultSet rs = conn.search(q);
 * </PRE>
 * <P>
 * Queries in ZOOM may take various forms: objects of this class
 * represent queries expressed in Index Data's Prefix Query Notation
 * (PQN), as implemented in their Z39.50 toolkit, Yaz.
 * <P>
 * As with all ZOOM Query types, there are no methods apart from the
 * constructor: the only thing you can do with a PrefixQuery is feed
 * it to <CODE>Connection.search()</CODE>.
 *
 * @version	$Id: PrefixQuery.java,v 1.2 2002/11/24 17:43:03 mike Exp $
 * @see	<A href="http://www.indexdata.dk/yaz/">The Yaz toolkit</A>
 * @see	<A href="http://www.indexdata.dk/yaz/doc/tools.php#AEN2194"
 *	>Prefix Query Notation</A>
 * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.3"
 *		       >zoom.z3950.org/api/zoom-1.1.html#3.3</A>
 */
public class PrefixQuery extends Query {
    private IRQuery q;

    /**
     * Creates a new Query object representing the specified Prefix
     * query.
     * 
     * @param pqn	The query itself expressed in PQN.
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.3.2"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.3.2</A>
     */
    public PrefixQuery(String pqn) {
	q = new IRQuery();
	// Default to top-level attribute set BIB-1
	String x = "@attrset bib-1 " + pqn;
	System.err.println("x=" + x);
	q.query = new PrefixString(x);
    }

    IRQuery _JZQuery() { return q; }
}
